@@ -44,10 +44,12 @@
 
 enum {
     PROP_0,
-    PROP_SOURCE_TAG,
-    PROP_SOURCE_LOCATION,
+    PROP_LABEL,
+    PROP_DESCRIPTION,
+    PROP_ICON,
     PROP_KEY_SERVER,
-    PROP_URI
+    PROP_URI,
+    PROP_ACTIONS
 };
 
 /* -----------------------------------------------------------------------------
@@ -59,10 +61,11 @@ struct _SeahorseServerSourcePrivate {
     gchar *uri;
 };
 
-static void seahorse_source_iface (SeahorseSourceIface *iface);
+static void      seahorse_server_source_collection_init    (GcrCollectionIface *iface);
 
-G_DEFINE_TYPE_EXTENDED (SeahorseServerSource, seahorse_server_source, G_TYPE_OBJECT, 0,
-                        G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_SOURCE, seahorse_source_iface));
+G_DEFINE_TYPE_WITH_CODE (SeahorseServerSource, seahorse_server_source, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (GCR_TYPE_COLLECTION, seahorse_server_source_collection_init);
+);
 
 /* GObject handlers */
 static void seahorse_server_source_finalize   (GObject *gobject);
@@ -88,9 +91,17 @@ seahorse_server_source_class_init (SeahorseServerSourceClass *klass)
     gobject_class->set_property = seahorse_server_set_property;
     gobject_class->get_property = seahorse_server_get_property;
 
-	g_object_class_override_property (gobject_class, PROP_SOURCE_TAG, "source-tag");
-	g_object_class_override_property (gobject_class, PROP_SOURCE_LOCATION, "source-location");
-	
+    /* These properties are used to conform to SeahorseSource, but are not actually used */
+    g_object_class_install_property (gobject_class, PROP_LABEL,
+            g_param_spec_string ("label", "Label", "Label", "", G_PARAM_READABLE));
+    g_object_class_install_property (gobject_class, PROP_DESCRIPTION,
+            g_param_spec_string ("description", "Description", "Description", "", G_PARAM_READABLE));
+    g_object_class_install_property (gobject_class, PROP_ICON,
+            g_param_spec_object ("icon", "icon", "Icon", G_TYPE_ICON, G_PARAM_READABLE));
+    g_object_class_install_property (gobject_class, PROP_ACTIONS,
+            g_param_spec_object ("actions", "actions", "Actions",
+                                 GTK_TYPE_ACTION_GROUP, G_PARAM_READABLE));
+
     g_object_class_install_property (gobject_class, PROP_KEY_SERVER,
             g_param_spec_string ("key-server", "Key Server",
                                  "Key Server to search on", "",
@@ -99,52 +110,7 @@ seahorse_server_source_class_init (SeahorseServerSourceClass *klass)
     g_object_class_install_property (gobject_class, PROP_URI,
             g_param_spec_string ("uri", "Key Server URI",
                                  "Key Server full URI", "",
-                                 G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
-    
-	seahorse_registry_register_function (NULL, seahorse_pgp_key_canonize_id, "canonize", SEAHORSE_PGP_STR, NULL);
-}
-
-static void
-seahorse_server_source_load_async (SeahorseSource *source,
-                                   GCancellable *cancellable,
-                                   GAsyncReadyCallback callback,
-                                   gpointer user_data)
-{
-	GSimpleAsyncResult *res;
-
-	res = g_simple_async_result_new (G_OBJECT (source), callback, user_data,
-	                                 seahorse_server_source_load_async);
-	g_simple_async_result_complete_in_idle (res);
-	g_object_unref (res);
-}
-
-static gboolean
-seahorse_server_source_load_finish (SeahorseSource *source,
-                                    GAsyncResult *result,
-                                    GError **error)
-{
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (source),
-	                      seahorse_server_source_load_async), FALSE);
-
-	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
-		return FALSE;
-
-	return TRUE;
-}
-
-/**
-* iface: The #SeahorseSourceIface to init
-*
-* Sets the load function in @iface
-*
-* This is the init function of the interface SEAHORSE_TYPE_SOURCE
-*
-**/
-static void 
-seahorse_source_iface (SeahorseSourceIface *iface)
-{
-	iface->load_async = seahorse_server_source_load_async;
-	iface->load_finish = seahorse_server_source_load_finish;
+                                 G_PARAM_READWRITE));
 }
 
 /**
@@ -205,7 +171,7 @@ seahorse_server_set_property (GObject *object, guint prop_id,
         g_return_if_fail (ssrc->priv->server && ssrc->priv->server[0]);
         break;
     case PROP_URI:
-        g_assert (ssrc->priv->uri == NULL);
+        g_free (ssrc->priv->uri);
         ssrc->priv->uri = g_strdup (g_value_get_string (value));
         g_return_if_fail (ssrc->priv->uri && ssrc->priv->uri[0]);
         break;
@@ -221,29 +187,64 @@ seahorse_server_set_property (GObject *object, guint prop_id,
 * pspec: ignored
 *
 * The properties that can be read are:
-* PROP_KEY_SERVER, PROP_URI, PROP_SOURCE_TAG, PROP_SOURCE_LOCATION
+* PROP_KEY_SERVER, PROP_URI
 *
 **/
 static void 
-seahorse_server_get_property (GObject *object, guint prop_id, GValue *value,
+seahorse_server_get_property (GObject *obj,
+                              guint prop_id,
+                              GValue *value,
                               GParamSpec *pspec)
 {
-    SeahorseServerSource *ssrc = SEAHORSE_SERVER_SOURCE (object);
-  
-    switch (prop_id) {
-    case PROP_KEY_SERVER:
-        g_value_set_string (value, ssrc->priv->server);
-        break;
-    case PROP_URI:
-        g_value_set_string (value, ssrc->priv->uri);
-        break;
-    case PROP_SOURCE_TAG:
-        g_value_set_uint (value, SEAHORSE_PGP);
-        break;
-    case PROP_SOURCE_LOCATION:
-        g_value_set_enum (value, SEAHORSE_LOCATION_REMOTE);
-        break;
-    }        
+	SeahorseServerSource *self = SEAHORSE_SERVER_SOURCE (obj);
+
+	switch (prop_id) {
+	case PROP_LABEL:
+	case PROP_KEY_SERVER:
+		g_value_set_string (value, self->priv->server);
+		break;
+	case PROP_DESCRIPTION:
+	case PROP_URI:
+		g_value_set_string (value, self->priv->uri);
+		break;
+	case PROP_ICON:
+		g_value_take_object (value, g_themed_icon_new (GTK_STOCK_DIALOG_QUESTION));
+		break;
+	case PROP_ACTIONS:
+		g_value_set_object (value, NULL);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
+		break;
+	}
+}
+
+static guint
+seahorse_server_source_get_length (GcrCollection *collection)
+{
+	return 0;
+}
+
+static GList *
+seahorse_server_source_get_objects (GcrCollection *collection)
+{
+	return NULL;
+}
+
+static gboolean
+seahorse_server_source_contains (GcrCollection *collection,
+                                 GObject *object)
+{
+	return FALSE;
+}
+
+static void
+seahorse_server_source_collection_init (GcrCollectionIface *iface)
+{
+	/* This is implemented because SeahorseSource requires it */
+	iface->get_length = seahorse_server_source_get_length;
+	iface->get_objects = seahorse_server_source_get_objects;
+	iface->contains = seahorse_server_source_contains;
 }
 
 /* --------------------------------------------------------------------------
@@ -366,3 +367,94 @@ seahorse_server_source_new (const gchar *server)
     g_free (uri);
     return ssrc;
 }
+
+void
+seahorse_server_source_search_async (SeahorseServerSource *self,
+                                     const gchar *match,
+                                     GcrSimpleCollection *results,
+                                     GCancellable *cancellable,
+                                     GAsyncReadyCallback callback,
+                                     gpointer user_data)
+{
+	g_return_if_fail (SEAHORSE_IS_SERVER_SOURCE (self));
+	g_return_if_fail (match != NULL);
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+	g_return_if_fail (SEAHORSE_SERVER_SOURCE_GET_CLASS (self)->search_async);
+	SEAHORSE_SERVER_SOURCE_GET_CLASS (self)->search_async (self, match, results,
+	                                                       cancellable, callback, user_data);
+}
+
+gboolean
+seahorse_server_source_search_finish (SeahorseServerSource *self,
+                                      GAsyncResult *result,
+                                      GError **error)
+{
+	g_return_val_if_fail (SEAHORSE_IS_SERVER_SOURCE (self), FALSE);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+	g_return_val_if_fail (SEAHORSE_SERVER_SOURCE_GET_CLASS (self)->search_finish, FALSE);
+	return SEAHORSE_SERVER_SOURCE_GET_CLASS (self)->search_finish (self, result, error);
+}
+
+void
+seahorse_server_source_export_async (SeahorseServerSource *self,
+                                     const gchar **keyids,
+                                     GCancellable *cancellable,
+                                     GAsyncReadyCallback callback,
+                                     gpointer user_data)
+{
+	SeahorseServerSourceClass *klass;
+
+	g_return_if_fail (SEAHORSE_IS_SERVER_SOURCE (self));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+	klass = SEAHORSE_SERVER_SOURCE_GET_CLASS (self);
+	g_return_if_fail (klass->export_async);
+	(klass->export_async) (self, keyids, cancellable, callback, user_data);
+}
+
+gpointer
+seahorse_server_source_export_finish (SeahorseServerSource *self,
+                                      GAsyncResult *result,
+                                      gsize *size,
+                                      GError **error)
+{
+	SeahorseServerSourceClass *klass;
+
+	g_return_val_if_fail (SEAHORSE_IS_SERVER_SOURCE (self), NULL);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);
+	g_return_val_if_fail (size != NULL, NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	klass = SEAHORSE_SERVER_SOURCE_GET_CLASS (self);
+	g_return_val_if_fail (klass->export_async != NULL, NULL);
+	g_return_val_if_fail (klass->export_finish != NULL, NULL);
+	return (klass->export_finish) (self, result, size, error);
+}
+
+void
+seahorse_server_source_import_async (SeahorseServerSource *source,
+                                     GInputStream *input,
+                                     GCancellable *cancellable,
+                                     GAsyncReadyCallback callback,
+                                     gpointer user_data)
+{
+	g_return_if_fail (SEAHORSE_IS_SERVER_SOURCE (source));
+	g_return_if_fail (G_IS_INPUT_STREAM (input));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+	g_return_if_fail (SEAHORSE_SERVER_SOURCE_GET_CLASS (source)->import_async);
+	SEAHORSE_SERVER_SOURCE_GET_CLASS (source)->import_async (source, input, cancellable,
+	                                                         callback, user_data);
+}
+
+GList *
+seahorse_server_source_import_finish (SeahorseServerSource *source,
+                                      GAsyncResult *result,
+                                      GError **error)
+{
+	g_return_val_if_fail (SEAHORSE_IS_SERVER_SOURCE (source), NULL);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+	g_return_val_if_fail (SEAHORSE_SERVER_SOURCE_GET_CLASS (source)->import_finish, NULL);
+	return SEAHORSE_SERVER_SOURCE_GET_CLASS (source)->import_finish (source, result, error);
+}
